package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmQuizDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQuestion;
import com.qunzhixinxi.hnqz.admin.mapper.WmQuestionMapper;
import com.qunzhixinxi.hnqz.admin.service.WmQuestionService;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 试题服务实现
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-10-16 16:01
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmQuestionServiceImpl extends ServiceImpl<WmQuestionMapper, WmQuestion> implements WmQuestionService {

	/**
	 * 校验试题
	 *
	 * @param operator   操作人
	 * @param label      标题
	 * @param questionId 试题ID
	 * @return 校验结果
	 */
	@Override
	public Boolean checkQuestion(HnqzUser operator, String label, Integer questionId) {

		int count = this.count(Wrappers.<WmQuestion>lambdaQuery()
				.eq(WmQuestion::getLabel, label)
				.eq(WmQuestion::getEnterpriseId, operator.getDeptId())
				.ne(questionId != null, WmQuestion::getQuestionId, questionId)
		);

		return count == 0;
	}

	/**
	 * 创建试题
	 *
	 * @param resource 试题信息
	 * @param operator 操作人
	 * @return 创建结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean createQuestion(WmQuizDTO.OnQuestionCreate resource, HnqzUser operator) {

		WmQuestion question = new WmQuestion();
		question.setSerialNumber(IdUtil.fastSimpleUUID());
		question.setLabel(resource.getLabel());
		question.setOptions(resource.getOptions());
		question.setAnswer(resource.getAnswer());
		question.setEnterpriseId(operator.getDeptId());
		String username = operator.getUsername();
		question.setCreateBy(username);
		question.setUpdateBy(username);
		LocalDateTime now = LocalDateTime.now();
		question.setCreateTime(now);
		question.setUpdateTime(now);

		return this.save(question);
	}

	/**
	 * 创建试题
	 *
	 * @param resource 试题信息
	 * @param operator 操作人
	 * @return 创建结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean updateQuestion(WmQuizDTO.OnQuestionUpdate resource, HnqzUser operator) {

		WmQuestion question = new WmQuestion();
		question.setQuestionId(resource.getQuestionId());
		question.setUpdateBy(operator.getUsername());
		question.setUpdateTime(LocalDateTime.now());

		if (StrUtil.isNotBlank(resource.getLabel())) {
			question.setLabel(resource.getLabel());
		}

		if (ArrayUtil.isNotEmpty(resource.getOptions())) {
			question.setOptions(resource.getOptions());
		}

		if (StrUtil.isNotBlank(resource.getAnswer())) {
			question.setAnswer(resource.getAnswer());
		}

		return this.updateById(question);
	}

	/**
	 * 获取试题分页
	 *
	 * @param current 当前页
	 * @param size    每页记录数
	 * @param entId   企业ID
	 * @param label   标题
	 * @return 分页结果
	 */
	@Override
	public Page<WmQuestion> pageQuestions(Integer current, Integer size, Integer entId, String label) {

		return this.page(new Page<>(current, size),
				Wrappers.<WmQuestion>lambdaQuery()
						.like(StrUtil.isNotBlank(label), WmQuestion::getLabel, label)
						.eq(entId != null, WmQuestion::getEnterpriseId, entId)
						.orderByDesc(WmQuestion::getUpdateTime)
		);
	}

	/**
	 * 获取试题详情
	 *
	 * @param questionId 试题ID
	 * @param sn         试题唯一序列号
	 * @return 试题信息
	 */
	@Override
	public WmQuestion getQuestionDetails(Integer questionId, String sn) {

		if (questionId != null) {
			return this.getById(questionId);
		} else if (StrUtil.isNotBlank(sn)) {
			return this.getOne(Wrappers.<WmQuestion>lambdaQuery().eq(WmQuestion::getSerialNumber, sn));
		} else {
			return null;
		}
	}

	/**
	 * 删除试题
	 *
	 * @param resource 试题信息
	 * @return 删除结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean deleteQuestion(WmQuizDTO.OnQuestionDel resource) {
		return this.removeById(resource.getQuestionId());
	}
}
