package com.qunzhixinxi.hnqz.admin.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qunzhixinxi.hnqz.admin.api.dto.WmQuizDTO;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQuiz;
import com.qunzhixinxi.hnqz.admin.api.entity.WmQuizItem;
import com.qunzhixinxi.hnqz.admin.mapper.WmQuizMapper;
import com.qunzhixinxi.hnqz.admin.service.WmQuizItemService;
import com.qunzhixinxi.hnqz.admin.service.WmQuizService;
import com.qunzhixinxi.hnqz.common.core.constant.enums.UpmsState;
import com.qunzhixinxi.hnqz.common.core.exception.BizException;
import com.qunzhixinxi.hnqz.common.security.service.HnqzUser;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 试卷服务实现
 *
 * @author jimmy
 * @version 1.0.0
 * @date 2023-10-16 15:55
 */
@Slf4j
@Service
@AllArgsConstructor
public class WmQuizServiceImpl extends ServiceImpl<WmQuizMapper, WmQuiz> implements WmQuizService {

	private final WmQuizItemService quizItemService;

	/**
	 * 获取试卷详情
	 *
	 * @param quizId 试卷ID
	 * @return 试卷详情
	 */
	@Override
	public Map<String, Object> getDetails(Integer quizId) {


		WmQuiz quiz = this.getById(quizId);

		List<WmQuizItem> quizItems = null;
		if (quiz != null) {
			quizItems = quizItemService.listItems(quiz.getSerialNumber());
		}

		Map<String, Object> res = new HashMap<>(2);
		res.put("quiz", quiz);
		res.put("items", CollUtil.isEmpty(quizItems) ? Collections.emptyList() : quizItems);
		return res;
	}

	/**
	 * 发布试题
	 *
	 * @param quizId   试题ID
	 * @param operator 操作人
	 * @return 发布结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean releaseQuiz(Integer quizId, HnqzUser operator) {
		return this.update(Wrappers.<WmQuiz>lambdaUpdate()
				.set(WmQuiz::getState, UpmsState.QuizState.RELEASE)
				.set(WmQuiz::getCreateBy, operator.getUsername())
				.set(WmQuiz::getUpdateTime, LocalDateTime.now())
				.eq(WmQuiz::getQuizId, quizId)
		);
	}

	/**
	 * 中止试卷
	 *
	 * @param quizId   试题ID
	 * @param operator 操作人
	 * @return 发布结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean stopQuiz(Integer quizId, HnqzUser operator) {
		return this.update(Wrappers.<WmQuiz>lambdaUpdate()
				.set(WmQuiz::getState, UpmsState.QuizState.STOP)
				.set(WmQuiz::getCreateBy, operator.getUsername())
				.set(WmQuiz::getUpdateTime, LocalDateTime.now())
				.eq(WmQuiz::getQuizId, quizId)
		);
	}

	/**
	 * 创建试卷
	 *
	 * @param resource 试卷信息
	 * @param user     操作人
	 * @return 创建结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public String createQuiz(WmQuizDTO.OnQuizCreate resource, HnqzUser user) {

		WmQuiz quiz = new WmQuiz();
		String sn = IdUtil.fastSimpleUUID();
		quiz.setSerialNumber(sn);
		quiz.setTitle(resource.getTitle());
		quiz.setIntroduction(resource.getIntroduction());
		quiz.setSourceType(resource.getSourceType());
		quiz.setState(UpmsState.QuizState.CREATE);
		quiz.setEnterpriseId(user.getDeptId());
		String username = user.getUsername();
		quiz.setCreateBy(username);
		quiz.setUpdateBy(username);
		LocalDateTime now = LocalDateTime.now();
		quiz.setCreateTime(now);
		quiz.setUpdateTime(now);

		return this.save(quiz) ? sn : "";
	}

	/**
	 * 更新试卷
	 *
	 * @param resource 试卷信息
	 * @param user     操作人
	 * @return 更新结果
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public Boolean updateQuiz(WmQuizDTO.OnQuizUpdate resource, HnqzUser user) {

		return this.update(Wrappers.<WmQuiz>lambdaUpdate()
				.set(StrUtil.isNotBlank(resource.getTitle()), WmQuiz::getTitle, resource.getTitle())
				.set(StrUtil.isNotBlank(resource.getIntroduction()), WmQuiz::getIntroduction, resource.getIntroduction())
				.set(resource.getExpand() != null, WmQuiz::getExpand, JSONUtil.toJsonStr(resource.getExpand()))
				.set(WmQuiz::getUpdateBy, user.getUsername())
				.set(WmQuiz::getUpdateTime, LocalDateTime.now())
				.eq(WmQuiz::getSerialNumber, resource.getSn())
		);
	}

	/**
	 * 校验试卷信息
	 *
	 * @param serialNum 序列号
	 * @param expand    试卷统计信息
	 */
	@Override
	public void checkQuizExpand(WmQuiz.Expand expand, String serialNum) {
		if (expand.getPassingMark() > expand.getTotalMark()) {
			log.error("通过分数不能大于总分");
			throw new BizException("通过分数不能大于总分");
		}

		List<WmQuizItem> items = quizItemService.listItems(serialNum);

		if (expand.getItemQty() != items.size()) {
			log.error("试卷项和参数值不符");
			throw new BizException("试卷项和参数值不符");
		}

		int sum = items.stream().mapToInt(WmQuizItem::getMark).sum();

		if (expand.getTotalMark() != sum) {
			log.error("总分和参数值不符");
			throw new BizException("总分和参数值不符");
		}

	}
}
